# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/postgres/access'
require 'easy_type'
require 'puppet/type/pg_schema_definition/version'
require 'fileutils'

Puppet::Type.type(:pg_schema_definition).provide(:psql) do
  include EasyType::Provider
  include EasyType::Template
  include EasyType::Helpers
  include Puppet_X::EnterpriseModules::Postgres::Access

  desc 'Manage The schema definition'

  mk_resource_methods

  def self.prefetch(resources)
    resources.each do |name, resource|
      options = options_for(resource[:schema_name], resource[:password], db_from(resource))
      destroy(resource) if resource[:reinstall].to_s == 'true'
      if version_table_exists?(options)
        statement = template('pg_config/pg_schema_definition/index.sql.erb', binding)
        versions = sql(statement, options)
      else
        versions = []
      end
      resource.provider = if versions.empty?
                            new(:ensure => :absent)
                          else
                            map_raw_to_resource(versions.last)
                          end
      next unless resource[:ensure].to_s == 'latest'

      # We need the sources here so
      resource.parameters[:source_path].process # fetch the files
      resource[:ensure] = resource.provider.latest_available_version
    end
  end

  def self.version_table_exists?(options)
    statement = template('pg_config/pg_schema_definition/version_table_exists.sql.erb', binding)
    rows = sql(statement, options)
    !rows.empty?
  end

  def self.options_for(role, password, db)
    options = { :db => db }
    options[:username] = role
    password = password.nil? ? role : password
    options[:password] = password
    options
  end

  def upgrade_to(version)
    copy_upgrade_scripts_to_tmp
    ensure_version_table
    scripts = scripts_for_upgrade(self.ensure, version)
    execute_scripts(scripts) do |script|
      sequence, application, version, description = parse_script_name(script)
      "insert into schema_version (id, application, version, description, installation_time) values ('#{sequence}', '#{application}', '#{version}', '#{description}', now())"
    end
  end

  def downgrade_to(version)
    copy_downgrade_scripts_to_tmp
    ensure_version_table
    scripts = scripts_for_downgrade(self.ensure, version)
    execute_scripts(scripts) do |script|
      sequence = parse_script_name(script)[0]
      "delete from schema_version where lpad(id::varchar,4,'0')='#{sequence}'"
    end
  end

  def self.destroy(resource)
    Puppet.info "#{resource.path}: deleting all schema information"
    options = options_for(resource[:schema_name], resource[:password], db_from(resource))
    options[:parse] = false
    statement = template('pg_config/pg_schema_definition/drop.sql.erb', binding)
    sql(statement, options)
  end

  def latest_available_version
    last_upgrade_script = upgrade_scripts.last
    if last_upgrade_script
      _, _, version = parse_script_name(upgrade_scripts.last)
    else
      version = :latest
    end
    version
  end

  private

  def copy_upgrade_scripts_to_tmp
    Pathname.glob("#{upgrade_path_for(resource[:source_path])}*.*").each { |f| FileUtils.cp(f.to_s, '/tmp') }
  end

  def copy_downgrade_scripts_to_tmp
    Pathname.glob("#{downgrade_path_for(resource[:source_path])}*.*").each { |f| FileUtils.cp(f.to_s, '/tmp') }
  end

  def execute_scripts(scripts)
    statements = template('pg_config/pg_schema_definition/execute.sql.erb', binding)
    statements << "select 1;\n" # This is needed when empty files are processed
    scripts.each do |script|
      statements << "-- \n"
      Puppet.info "#{resource.path}: Applying #{script}"
      statements << "\\echo Executing file #{script}\n"
      statements << "-- \n"
      statements << "begin;\n"
      statements << File.read(script)
      statements << "commit;\n"
      # TODO: When we support connect strins refacter
      # connect_string = configuration_value_for(resource[:db], 'connect_string')
      # statements << if connect_string.empty?
      #                 "set role #{resource[:schema_name]};\n"
      #               else
      #                 # TODO: connect string management
      #                 "set role #{resource[:schema_name]};\n"
      #               end
      statements << "set role #{resource[:schema_name]};\n"
      statements << yield(script) << ";\n"
      statements << "commit;\n"
    end
    options = self.class.options_for(resource[:schema_name], resource[:password], db_from(resource))
    options[:parse] = false
    FileUtils.cd(resource[:source_path]) do
      sql statements, options
    end
  end

  def scripts_for_upgrade(from, to)
    unloaded_scripts(upgrade_scripts.select { |s| required_for_upgrade?(s, from, to) }.sort!)
  end

  def scripts_for_downgrade(from, to)
    downgrade_scripts.select { |s| required_for_downgrade?(s, from, to) }.sort!.reverse!
  end

  def required_for_upgrade?(script_name, source_from, source_to)
    required_for?(script_name, source_from, source_to) { |version, from, to| version > from && version <= to }
  end

  def required_for_downgrade?(script_name, source_from, source_to)
    required_for?(script_name, source_from, source_to) { |version, from, to| version <= from && version > to }
  end

  def required_for?(script_name, from, to)
    _, _, version = parse_script_name(script_name)
    begin
      version_object = Version.new(version)
      yield(version_object, from, to)
    rescue ArgumentError => e
      fail "#{e.message}; Check file name format of #{script_name}."
    end
  end

  def parse_script_name(name)
    name.basename.to_s.scan(/^(\d{4}).*_(\D*).*_(\d+.\d+.\d+.*).*_(.*)\.sql$/).flatten
  end

  def unloaded_scripts(scripts)
    scripts.reject do |script|
      id, = parse_script_name(script)
      loaded?(id)
    end
  end

  def upgrade_scripts
    Pathname.glob("#{upgrade_path_for(resource[:source_path])}*.sql").sort
  end

  def downgrade_scripts
    Pathname.glob("#{downgrade_path_for(resource[:source_path])}*.sql").sort
  end

  def loaded?(id)
    options = self.class.options_for(resource[:schema_name], resource[:password], db_from(resource))
    !sql("select * from schema_version where id=#{id}", options).empty?
  end

  def ensure_version_table
    options = self.class.options_for(resource[:schema_name], resource[:password], db_from(resource))
    create_version_table(options) unless self.class.version_table_exists?(options)
  end

  def create_version_table(options)
    Puppet.info "#{resource.path}: Creating schema_versions table."
    options[:parse] = false
    statement = template('pg_config/pg_schema_definition/create_table_schema_version.sql.erb', binding)
    sql(statement, options)
  end

  def upgrade_path_for(source_path)
    paths = Dir.glob("#{source_path}/**/upgrades")
    fail "Only one upgrade directory allowed in source_pad. #{paths.count} found." if paths.count > 1
    fail "source_path #{resource[:source_path]} doesn't contain a upgrades folder" if paths.empty?

    "#{paths.first}/"
  end

  def downgrade_path_for(source_path)
    paths = Dir.glob("#{source_path}/**/downgrades")
    fail "Only one downgrade directory allowed in source_pad. #{paths.count} found." if paths.count > 1
    fail "source_path #{resource[:source_path]} doesn't contain a downgrades folder" if paths.empty?

    "#{paths.first}/"
  end
end
