# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'easy_type/array_property'
require 'puppet_x/enterprisemodules/postgres/access'
require 'puppet_x/enterprisemodules/postgres/read_only_property'
require 'puppet_x/enterprisemodules/postgres/title_parser'
require 'puppet_x/enterprisemodules/postgres/utilities'

#
# Create a new type pg_database.
#
Puppet::Type.newtype(:pg_database) do
  include EasyType
  include Puppet_X::EnterpriseModules::Postgres::Access
  include Puppet_X::EnterpriseModules::Postgres::Utilities
  extend Puppet_X::EnterpriseModules::Postgres::TitleParser

  desc <<-DOC
    With this type you can create,delete and manage Postgres databases. Here is an example
    on how you could use this:

        pg_database { 'my_database@localhost':
          ensure            => 'present',
          allow_connections => true,
          connection_limit  => -1,
          owner             => 'my_custom_role,
        }

    The `pg_database` type recognises all of the options you also have when using SQL to create a user or a role.
  DOC

  ensurable

  to_get_raw_resources do
    sql_on_all_databases <<-SQL
      select
        db.oid as oid,
        db.datctype as dactype,
        db.datcollate as datcollate,
        db.datallowconn as datallowconn,
        db.datconnlimit as datconnlimit,
        db.datistemplate as datistemplate,
        db.datname as datname,
        auth.rolname as rolname,
        ts.spcname as ts_name
      from pg_database db, pg_authid auth, pg_tablespace ts
      where
        db.datdba = auth.oid
        and db.dattablespace = ts.oid
    SQL
  end

  on_create do
    [:sql, "create database #{db_name}", { :db => db }]
  end

  on_modify do
    [:sql, "alter database #{db_name}", { :skip_without_properties => true, :db => db }]
  end

  before_destroy do
    sql("alter database #{db_name} is_template false", { :db => db }) if provider.is_template.to_s == 'true'
  end

  on_destroy do
    [:sql, "drop database #{db_name}", { :db => db }]
  end

  map_title_to_db(:db_name)

  parameter :name
  parameter :db_name
  parameter :db
  parameter :template

  property :oid
  property :owner
  property :locale
  property :lc_collate
  property :lc_ctype
  property :tablespace
  property :allow_connections
  property :connection_limit
  property :is_template
end
